<#
.SYNOPSIS
    A sample script to demonstrate how deployments of Autodesk Desktop Connector can be done.

.DESCRIPTION
    This script demonstrates how Desktop Connector can deployed .
    It is intended as a reference for IT admins for Microsoft Configuration Manager (formerly SCCM) deployments.
    Users should not directly run this script without first reading through it themselves and making changes where needed.
    This script supports install, upgrade, and uninstall operations.
    For install workflows, this script should be ran in the same directory as the Setup.exe executable.
    Refer to the online help for more information

.PARAMETER DeploymentType
    The type of deployment to perform. Options include Install and Uninstall
    Note that install can also be used for upgrade workflows

.EXAMPLE
    C:\PS> .\Sample-Only-Deployment-Script.ps1 -DeploymentType Install

.EXAMPLE
    C:\PS> .\Sample-Only-Deployment-Script.ps1 -DeploymentType Uninstall

.LINK
    https://help.autodesk.com/view/CONNECT/ENU/

.NOTES
    Version: 3
    Date:    3/24/2023  
#>


param(
    # The type of deployment to perform.
    # Aliases: D
    [Alias("D")]
    [String]$DeploymentType
)

function ShutdownDesktopConnector
{
    $processName = "DesktopConnector.Applications.Tray"
    if (Get-Process $processName -ErrorAction SilentlyContinue)
    {
        Stop-Process -Name $processName -Force
    }
}

function GetInstalledVersion
{
    $appName = "Autodesk Desktop Connector"
    $installEntries = Get-ItemProperty 'HKLM:\Software\Microsoft\Windows\CurrentVersion\Uninstall\*' | Select-Object DisplayName, DisplayVersion, UninstallString | Where-Object { $_.DisplayName -match "^*$appname*"} | Where-Object { $_.UninstallString -match "^*ODIS*"}
    
    if ($installEntries.Length -eq 0)
    {
        return $null
    }
    return $installEntries[0]
}

function Uninstall($RegistryEntry)
{
    $uninstallString = $RegistryEntry.UninstallString
    $splitUninstallString = $uninstallString -split ".exe"
    $uninstallExe = "$($splitUninstallString[0]).exe"
    $arguments = "$($splitUninstallString[1]) -q"
    $process = Start-Process -FilePath $uninstallExe -Wait -PassThru -ArgumentList $arguments
    if ($process.ExitCode -ne 0)
    {
        Write-Host "Encountered error while running uninstall!" -ForegroundColor Red
        exit 1
    }
}

function UninstallPreV16VersionIfExists
{
    Write-Host "Checking for existing installs of Desktop Connector..."
    
    # Search the registry for an installed version of Desktop Connector
    $installedVersion = GetInstalledVersion
    if ($installedVersion -eq $null)
    {
        Write-Host "No other version of DC is installed on this machine, continuing..."
        return
    }

    $version = $installedVersion.DisplayVersion
    $isPreV16 = $version.StartsWith("14.") -or $version.StartsWith("15.")

    if ($isPreV16 -eq $true)
    {
        Write-Host "Pre 16.X version of DC found on machine. This script will uninstall this so that it may continue..."

        ######################################################################################################################
        #                                                     !IMPORTANT!                                                    #
        ######################################################################################################################
        # To migrate from an earlier install to a 16.X install, the user's local workspace should be cleaned out
        # If there is any unsynced data on the user's machine in these locations, this may wipe it out
        Write-Host "Clearing out old local workspace locations for users..." -ForegroundColor Yellow
        $profilePaths = Get-ChildItem 'HKLM:\Software\Microsoft\Windows NT\CurrentVersion\ProfileList' | ForEach-Object { $_.GetValue('ProfileImagePath') }
        
		ForEach ($profilePath in $profilePaths)
        {
            RemoveDirectoryRecursive([io.path]::Combine($profilePath, "ACCDocs"))
            RemoveDirectoryRecursive([io.path]::Combine($profilePath, "ADrive"))
            RemoveDirectoryRecursive([io.path]::Combine($profilePath, "BIMTeam"))
            RemoveDirectoryRecursive([io.path]::Combine($profilePath, "Fusion"))
        }

        Write-Host "Uninstalling version $version..."
        Uninstall $installedVersion

        Write-Host "Old version uninstalled, continuing with installation of new version..."
    }
    else
    {
        Write-Host "Existing version $version found installed; upgrading..."
    }
}

# Adapted from: https://stackoverflow.com/a/39587889/495262
function RemoveDirectoryRecursive($directory)
{
    # create a temporary (empty) directory
    $parent = [System.IO.Path]::GetTempPath()
    [string] $name = [System.Guid]::NewGuid()
    $tempDirectory = New-Item -ItemType Directory -Path (Join-Path $parent $name)

    try
    {
        robocopy /MIR $tempDirectory.FullName $directory | out-null
        Remove-Item $directory -Force -Recurse | out-null
    }
    finally
    {
        Remove-Item $tempDirectory -Force -Recurse | out-null
    }
}

$ErrorActionPreference = "Stop"

Write-Host "Running $($MyInvocation.MyCommand.Path)..."

# If no arguments are provided, show the help
if($args.Count -eq 0 -and $DeploymentType -eq "")
{
    Get-Help $MyInvocation.MyCommand.Definition -full
    return
}

Write-Host "This script is provided as a sample only." -ForegroundColor Yellow
Write-Host "We recommend that IT admins use this script only for reference when writing their own script." -ForegroundColor Yellow
Write-Host "Exiting..." -ForegroundColor Yellow
exit 1

if ($DeploymentType -eq "Install")
{
    # Compute the path to the installer exe and check that it exists
    $pathToInstaller = [io.path]::Combine($PSScriptRoot, "Setup.exe")
    $installerExists = Test-Path $pathToInstaller
    if ($installerExists -eq $false)
    {
        
        Write-Host "An installer for was not found at $pathToInstaller" -ForegroundColor Red
        Write-Host "This script must be placed inside the same directory as the Setup.exe" -ForegroundColor Yellow
        exit 1
    }

    Write-Host "Installer found at $pathToInstaller..."

    Write-Host "Shutting down any running instances of Desktop Connector..."
    ShutdownDesktopConnector

    UninstallPreV16VersionIfExists

    Write-Host "Running installer..."
    $process = Start-Process -FilePath $pathToInstaller -Wait -PassThru -ArgumentList "-i install -q"
    if ($process.ExitCode -ne 0)
    {
        Write-Host "Encountered error while running installer!" -ForegroundColor Red
        exit 1
    }

    Write-Host "Installation complete"
    exit 0
}
elseif ($DeploymentType -eq "Uninstall")
{
    Write-Host "Checking for installed versions of Desktop Connector..."

    # Search the registry for an installed version of Desktop Connector
    $installedVersion = GetInstalledVersion
    if ($installedVersion -eq $null)
    {
        Write-Host "No versions of Autodesk Desktop Connector were found installed on this machine" -ForegroundColor Yellow
        exit 0
    }

    Write-Host "Detected installed version of Desktop Connector with version $($installedVersion.DisplayVersion)..."

    Write-Host "Shutting down any running instances of Desktop Connector..."
    ShutdownDesktopConnector

    # Obtain the uninstall string and execute the process
    Write-Host "Uninstalling Desktop Connector with version $($installedVersion.DisplayVersion)..."
    Uninstall $installedVersion

    Write-Host "Desktop Connector uninstalled"
}
else
{
    Write-Host "Invalid DeploymentType specified; valid options include 'Install' and 'Uninstall'" -ForegroundColor Red
    exit 1
}